package edu.travel.cache.util;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import java.util.concurrent.TimeUnit;

@Component
public class RedisUtil {
    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    /**
     * 通过KEY 获取值
     */
    public String getString(String key) {
        return stringRedisTemplate.opsForValue().get(key);
    }

    /**
     * 设置KEY 和 VALUE
     */
    public void setString(String key, String value) {
        stringRedisTemplate.opsForValue().set(key, value);
    }

    /**
     * @param key      KEY
     * @param value    值
     * @param expire   过期时间
     * @param timeUnit 时间单位
     */
    public void setString(String key, String value, long expire, TimeUnit timeUnit) {
        stringRedisTemplate.opsForValue().set(key, value, expire, timeUnit);
    }


    // 延迟删除（异步）
    @Async("taskExecutor")
    public void deleteDelay(String key, long delay, TimeUnit unit) {
        try {
            Thread.sleep(unit.toMillis(delay));
            stringRedisTemplate.delete(key);
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
        }
    }
}
